/**
  * This is an install.js file that does the following --
  *
  * 1. Installs to the current browser that is invoking the installation
  *
  * 2. Additionally installs to a secondary location on the Windows desktop,
  *    in this case C:\WINNT\System32\MyPlugin\
  *
  * 3. Writes registry keys exposing the above secondary install location for
  *    other browsers to find.  The keys are written according to the specification:
  *    http://mozilla.org/projects/plugins/first-install-problem.html and follows the
  *    PLID specification: http://mozilla.org/projects/plugins/plugin-identifier.html
  *
 */

// Name of the files to be installed
var PLUGIN_FILE    = "NPMCult3DP.dll";
var COMPONENT_FILE = "nsIC3DPScriptablePeer.xpt";

//!!! VERY IMPORTANT TO UPDATE
var CULT_MAJOR_VER = "5.3";                      // Major minor version
var CULT_VERSION   = CULT_MAJOR_VER + ".0.154";  // Append the buildnr to the major version

//!!! VERY IMPORTANT TO UPDATE
// The plug-in sise in kilobyte, both the dll and the xpi file. Used to check if there is enough space left on the clients harddisk
var PLUGIN_SIZE    = 2000; // (DLL file) Reserve a little extra so it is not required to update too often
var COMPONENT_SIZE = 10;   // (XPI file) Reserve a little extra so it is not required to update too often


// Cycore's own Error codes
var exceptionOccuredError       = -4711;
var winRegIsNullError           = -4712;
var invalidRootkeyError         = -4713;
var registrykeyNotWritableError = -4714;
var noSecondaryInstall          = -4715;

// Variables to handle firstTime installation problem
// Cult3D's unique PLID (Plugin Identifier)
var CULT_PLID = "@cycore.com/Cult3DViewer";


// Registry constant path's
var HKEY_LOCAL_MACHINE = "HKEY_LOCAL_MACHINE";
var HKEY_CURRENT_USER  = "HKEY_CURRENT_USER";

var REG_MOZ_PATH = "SOFTWARE\\MozillaPlugins";

// Descriptions
var PLUGIN_DESCRIPTION = "<A HREF=\"http://www.cult3d.com\">Cult3D</A> Plug-in "+CULT_VERSION+" from <A HREF=\"http://www.cycore.com\">Cycore AB</A>";

var VENDOR = "Cycore";
var COMPANY_NAME = VENDOR;

var PRODUCT_NAME = "Cult3D Viewer";

var CO_MIME_TYPE = "application/x-cult3d-object";
var CO_SUFFIX = "co";
var CO_SUFFIX_DESCRIPTION = "Cult3D files";


var err;


////////////////////////////////////////
//         MAIN                       //
//         main                       //
//        M A I N                     //
////////////////////////////////////////
c3dMain();

function c3dMain()
{
	try
	{
		var installationSuccessfull = true;
		var exceptionType = "";


		err = initInstall(PRODUCT_NAME, CULT_PLID, CULT_VERSION);
		logComment("C3DMoz initInstall: " + err);
		if (err != 0) /// InitInstall failed. Couldn't even start to initialize the installation
		{
			logComment("C3DMoz Error, initInstall failed "+err+". Data[ProductName="+PRODUCT_NAME+", PLID="+CULT_PLID+", Version="+CULT_VERSION+"]");
			cancelInstall(err);
			refreshPlugins();
            resetError();

			return err;
		}

		// Adding files to be installed to _this_ browser
		// (this browser is the browser which triggered the installation of Cult3D plug-in)
		err = startC3DInstallation();
		if (err != 0)  // Installs the plug-in for the current gecko browser(s)
		{
			logComment("C3DMoz startC3DInstallation failed. Errorcode="+error);
			cancelInstall(err);
			refreshPlugins();
			resetError();

			return err;
		}

		// Start the solution for "First-Install problem"
		// Write to a well known place in the registry info about our plug-in
		err = createSecondaryInstall();
		if (err != 0)
		{
			logComment("C3DMoz createSecondaryInstall failed. Errorcode="+err);
			logComment("C3DMoz continuing installing plug-in");
		}

		///////////////////////////////////////////////////
		/////// Performing the actual installation     ///
		/////////////////////////////////////////////////

		logComment("C3DMoz performing installation");
		resetError();
		err = performInstall();
		logComment("C3DMoz plugin performInstall() returned: " + err);
		if (err == REBOOT_NEEDED)
		{
			var errMsg = "You need to restart your browser in order for the installation to complete";
			logComment("C3DMoz "+errMsg);
		}

		if ((err != REBOOT_NEEDED) && (err != SUCCESS))
		{
			var errMsg = "Installation of Cult3D failed. (performInstall) ErrorCode="+err;
			logComment("C3DMoz "+errMsg);
			cancelInstall(err);
			refreshPlugins();
			resetError();

			return err;
		}

		var succMsg = "Installation of Cult3D plug-in succeeded";
		logComment("C3DMoz "+succMsg);

		logComment("C3DMoz refreshing plugin info");
		refreshPlugins();

		logComment("C3DMoz the installation seems to be successfull. Errorcode = "+getLastError());
	}
	catch(e)
	{
		var foo = getLastError();
		logComment("C3DMoz the installation failed with exception, message="+e.message+" ErrorCode="+foo);
		cancelInstall(exceptionOccuredError);
		refreshPlugins();
		resetError();

		return exceptionOccuredError;
	}
}


/**
 * This function installs the Mozilla Cult3D plugins.
 *
 * @return The value of 0, if the installation were successfull, non-zero otherwise.
 */
function startC3DInstallation()
{
	var pluginsFolder    = getFolder("Plugins");
	var componentsFolder = getFolder("Components");

	logComment("C3DMoz pluginsFolder:    " + pluginsFolder);
	logComment("C3DMoz componentsFolder: " + componentsFolder);
	if(verifyDiskSpace(pluginsFolder, PLUGIN_SIZE+COMPONENT_SIZE) == false)
	{
		// Insufficent disk space to install the plug-in
		var errMsg = "Not enough space on harddisk to install the plug-in";
		logComment("C3DMoz "+errMsg);
		logComment("C3DMoz   -- Plugin folder diskspace:    "+getAvailableDiskspace(pluginsFolder));
		logComment("C3DMoz   -- Component folder diskspace: "+getAvailableDiskspace(componentsFolder));

		return INSUFFICIENT_DISK_SPACE;
	}
	logComment("C3DMoz enough space on harddisk to install the plugin");

	// start installing plugin to shared library
	resetError();

	if (!fileExists(pluginsFolder))
	{
		err = dirCreate(pluginsFolder);
		if (err != 0)
		{
			var errMsg = "Could not create the missing plug-in directory("+pluginsFolder+"). Errorcode "+err;
			logComment("C3DMoz "+errMsg);

			return err;
		}
	}
	else
	{
		logComment("C3DMoz Plugins folder("+pluginsFolder+") already exists");
	}

	if (!fileExists(componentsFolder))
	{
		err = dirCreate(componentsFolder);
		if (err != 0)
		{
			var errMsg = "Could not create the missing component directory("+componentsFolder+"). Errorcode "+err;
			LogComment("C3DMoz "+errMsg);

			return err;
		}
	}
	logComment("C3DMoz Components folder("+componentsFolder+") already exists");

	err = addFile(CULT_PLID, CULT_VERSION, PLUGIN_FILE, pluginsFolder, null);
	if (err != 0)
	{
		logComment("C3DMoz adding file "+PLUGIN_FILE+" failed. Error code: " + err);

		return err;
	}

	err = addFile(CULT_PLID, CULT_VERSION, COMPONENT_FILE, componentsFolder, null);
	if (err != 0)
	{
		logComment("C3DMoz adding file "+COMPONENT_FILE+" failed. Error code: " + err);

		return err;
	}
	return 0;
} // startInstallC3DPlugins


/**
 * this function verifies disk space in kilobytes
  *
  * @param dirPath        The folder where to check if there is enough space
  * @param spaceRequired  The minimum pace that must be avaible for this test to be true
  *
  * @return true if there is enough diskspace return@mycompany.com/MyApplication,version=5.01
  * @return false if there isn't enough disk space
  */
function verifyDiskSpace(dirPath, spaceRequired)
{
	// Get the available disk space on the given path
	var spaceAvailable = fileGetDiskSpaceAvailable(dirPath);

    // Convert the available disk space into kilobytes
	spaceAvailable = parseInt(spaceAvailable / 1024);

    // do the verification
	if(spaceAvailable < spaceRequired)
	{
		logComment("C3DMoz Insufficient disk space: " + dirPath);
		logComment("C3DMoz   required : " + spaceRequired + " K");
		logComment("C3DMoz   available: " + spaceAvailable + " K");

		return false;
	}

	return true;
}

/**
 * This function returns how many kilobytes are free in the specified folder
 *
 * @param dirPath  The folder where to check if there is enough diskspace
 *
 * @return The amount of free diskspaces, expressed in kilobytes
 */
function getAvailableDiskspace(dirPath)
{
	var spaceAvailable = fileGetDiskSpaceAvailable(dirPath);
	return parseInt(spaceAvailable / 1024);
}

/**
 * Function for secondary installation of plugin (FirstInstall).
 * You should not stop the install process because the function failed,
 * you still have a chance to install the plugin for the already
 * installed gecko browsers.
 */
function createSecondaryInstall()
{
	// Use getFolder in such a way that it creates C:\WINNT\System32\Cult3D
	var secondaryFolder = getFolder("Win System", "Cult3D");

	// if secondaryFolder is NULL, then there has been an error
	if(secondaryFolder == null)
	{
		return noSecondaryInstall;
	}

	// we have admin privileges to write to the Win System directory
	// so we will set up DLL and XPT in their new home
	err = addFile (CULT_PLID, CULT_VERSION, PLUGIN_FILE, secondaryFolder, null);

	//  Something went wrong if err is NOT 0
	if (err != 0)
	{
		logComment("C3DMoz Could NOT add " + PLUGIN_FILE + " to " + secondaryFolder + ":" + err);
		return err;
	}

	// start installing xpt file if this is a scriptable plugin
	err = addFile (CULT_PLID, CULT_VERSION, COMPONENT_FILE, secondaryFolder, null);
	if (err != 0)
	{
		logComment("C3DMoz Could NOT add " + COMPONENT_FILE + " to " + secondaryFolder + ":" + err);
		return err;
	}

///////////////////////
// WritePlidSolution //
//////////////////////

	//Concatenate the secondary install path with the filename to make a fully qualified pathname
	var qualifiedSecondaryFolderDLL = secondaryFolder + PLUGIN_FILE;
	var qualifiedSecondaryFolderXPT = secondaryFolder + COMPONENT_FILE;

	// write PLID keys (mozilla.org/projects/plugins/first-install-problem.html)
	// write PLID keys to HKLM
	var HKLM_status =  registerPLID(HKEY_LOCAL_MACHINE, REG_MOZ_PATH,
									CULT_PLID,
					 		   		qualifiedSecondaryFolderDLL, qualifiedSecondaryFolderXPT,
					 		   		PLUGIN_DESCRIPTION, COMPANY_NAME, PRODUCT_NAME, CULT_VERSION,
					 		   		CO_MIME_TYPE, CO_SUFFIX, CO_SUFFIX_DESCRIPTION);

	logComment("C3DMoz First Install Installation: registerPLID("+HKEY_LOCAL_MACHINE+") returned, status "+HKLM_status);

	if (HKLM_status == false)
	{
		// write PLID keys (mozilla.org/projects/plugins/first-install-problem.html)
		// write PLID keys to HKCU
		var HKCU_status =  registerPLID(HKEY_CURRENT_USER, REG_MOZ_PATH,
										CULT_PLID,
					 		   			qualifiedSecondaryFolderDLL, qualifiedSecondaryFolderXPT,
					 		   			PLUGIN_DESCRIPTION, COMPANY_NAME, PRODUCT_NAME, CULT_VERSION,
					 		   			CO_MIME_TYPE, CO_SUFFIX, CO_SUFFIX_DESCRIPTION);


		logComment("C3DMoz First Install Installation: registerPLID("+HKEY_CURRENT_USER+") returned, status "+HKLM_status);

		if (HKCU_status != 0)
		{
        	logComment("C3DMoz Could not write to the registry. Errorcode="+HKCU_status);
        	return HKCU_status;
		}
	}


	return 0; // 0 means everything went well with the secondary install
}

/**
 * Function for preinstallation of plugin (FirstInstall).
 * You should not stop the install process because the function failed,
 * you still have a chance to install the plugin for the already
 * installed gecko browsers.
 *
 * @param rootKey   must be one of these two string values HKEY_LOCAL_MACHINE or HKEY_CURRENT_USER
 * @param plidID    Cycore's home-made "unique" ID for Cult3D viewers
 * @param plidPath  the registry path "SOFTWARE\MozillaPlugins"
 *
 * @param dllAbsolutePath   the fully qualified path to the DLL
 * @param xptAbsolutePath   the fully qualified path to the XPT
 * @param pluginDescription a String describing the plugin
 * @param vendor            a String describing the vendor
 * @param productName       the name of this software
 * @param pluginVersion     version string of the plugin
 * @param mimeType          MIME type handled by the plugin. Plugins with more than one MIME type might use an array
 * @param suffix            Suffix handled by plugin (e.g. .co)
 * @param suffixDescription String describing suffix
 */
function registerPLID(rootKey, plidPath,
					  plidID,
					  dllAbsolutePath, xptAbsolutePath,
					  pluginDescription, vendor, productName, pluginVersion,
					  mimeType, suffix, suffixDescription)
{
	var c3dRegStatus = 0;

	winreg = getWinRegistry();

	if (winreg == null)
	{
		logComment("C3DMoz registerPLID: winreg == null");
		return winregIsNullError;
	}

	// Which root to start from HKLM, HKCU
	if (rootKey == HKEY_LOCAL_MACHINE)
	{
		logComment("C3DMoz registerPLID: rootKey=="+HKEY_LOCAL_MACHINE);
		winreg.setRootKey(winreg.HKEY_LOCAL_MACHINE);
	}
	else if (rootKey == HKEY_CURRENT_USER)
	{
		logComment("C3DMoz registerPLID: rootKey=="+HKEY_CURRENT_USER);
		winreg.setRootKey(winreg.HKEY_CURRENT_USER);
	}
	else
	{
		logComment("C3DMoz registerPlid: invalid rootkey, "+rootKey);
		return invalidRootkeyError;
	}

	if (!winreg.isKeyWritable(plidPath))
	{
		logComment("C3DMoz registerPLID: registry key not writable");
		return registryKeyNotWritableError;
	}

	// If we can't find the plidPath create the key
	if (!winreg.keyExists(plidPath))
	{
		logComment("C3DMoz registerPLID: creating missing key "+plidPath+".");
		c3dRegStatus = winreg.createKey(plidPath, "");
		if (c3dRegStatus != 0)
		{
			logComment("C3DMoz registerPLID: Could not create the key, "+plidPath+"as expected. Errorcode="+c3dRegStatus);
			return c3dRegStatus;
		}
	}

	// OK were done, let's write info about our plugin, path, productname, etc


	// Write a subkey to MozillaPlugins with our PLID
	//var c3dSetStrRC = winreg.setValueString(plidPath, plidID, dllAbsolutePath);

	c3dMozillaPluginPath = plidPath+"\\"+plidID; // For instance. SOFTWARE\MozillaPlugins\@Cycore.com/Cult3DViewer
	c3dRegStatus = winreg.createKey(plidPath+"\\"+plidID, "");
	if (c3dRegStatus != 0)
	{
		logComment("C3DMoz registerPLID: could not create the subkey "+plidID+" to "+plidPath+". Errorcode="+c3dRegStatus);
		return c3dRegStatus;
	}


	// Write path to DLL
	c3dRegStatus = winreg.setValueString(c3dMozillaPluginPath, "Path", dllAbsolutePath);
	if (c3dRegStatus != 0)
	{
		logComment("C3DMoz registerPLID: Could not write the DLL path value. RegPath="+c3dMozillaPluginPath+", DLLPath="+dllAbsolutePath+". ErrorCode="+c3dRegStatus);
		return c3dRegStatus;
	}

	// Write XPTPath
	c3dRegStatus = winreg.setValueString(c3dMozillaPluginPath, "XPTPath", xptAbsolutePath);
	if (c3dRegStatus != 0)
	{
		logComment("C3DMoz registerPLID: Could not write the XPT path value. RegPath="+c3dMozillaPluginPath+"XPTPath="+xptAbsolutePath+", as expected. Errorcode="+c3dRegStatus);
		return c3dRegStatus;
	}

	// Write the productName
	c3dRegStatus = winreg.setValueString(c3dMozillaPluginPath, "ProductName", productName);
	if (c3dRegStatus != 0)
	{
		logComment("C3DMoz registerPLID: Could not write productName. RegPath="+c3dMozillaPluginPath+", productName="+productName+", as expected. Errorcode="+c3dRegStatus);
		return c3dRegStatus;
	}

	// Write the vendor
	c3dRegStatus = winreg.setValueString(c3dMozillaPluginPath, "Vendor", vendor);
	if (c3dRegStatus != 0)
	{
		logComment("C3DMoz registerPLID: Could not write vendorName. RegPath="+c3dMozillaPluginPath+", vendorName="+vendor+", as expected. Errorcode="+c3dRegStatus);
		return c3dRegStatus;
	}

	// Write the plugin description
	c3dRegStatus = winreg.setValueString(c3dMozillaPluginPath, "Description", pluginDescription);
	if (c3dRegStatus != 0)
	{
		logComment("C3DMoz registerPLID: Could not write plugin description. RegPath="+c3dMozillaPluginPath+", description="+pluginDescription+", as expected. Errorcode="+c3dRegStatus);
		return c3dRegStatus;
	}

	// Write the version number of the plug-in
	c3dRegStatus = winreg.setValueString(c3dMozillaPluginPath, "Version", pluginVersion);
	if (c3dRegStatus != 0)
	{
		logComment("C3DMoz registerPLID: Could not write plugin version. RegPath="+c3dMozillaPluginPath+", pluginVersion="+pluginVersion+", as expected. Errorcode="+c3dRegStatus);
		return c3dRegStatus;
	}

	/////////////////////////////////
	// Write the subkey MimeTypes //
	///////////////////////////////
	var c3dMimetypePath = c3dMozillaPluginPath+"\\MimeTypes";
	c3dRegStatus = winreg.createKey(c3dMimetypePath, "");
	if (c3dRegStatus != 0)
	{
		logComment("C3DMoz registerPLID: Could not create the subkey MimeTypes, "+c3dMimetypePath+" as expected. Errorcode="+c3dRegStatus);
		return c3dRegStatus;
	}

	var coMimetypePath = c3dMimetypePath+"\\"+mimeType;
	c3dRegStatus = winreg.createKey(coMimetypePath, "");
	if (c3dRegStatus != 0)
	{
		logComment("C3DMoz registerPLID: Could not create the subkey "+coMimetypePath+", as expected. Errorcode="+c3dRegStatus);
		return c3dRegStatus;
	}

	// Write the description of the co mimetype
	c3dRegStatus = winreg.setValueString(coMimetypePath, "Description", suffixDescription);
	if (c3dRegStatus != 0)
	{
		logComment("C3DMoz registerPLID: Could not create the suffix description value, "+suffixDescription+", as expected. Errorcode="+c3dRegStatus);
		return c3dRegStatus;
	}

	c3dRegStatus = winreg.setValueString(coMimetypePath, "Suffixes", suffix)
	if (c3dRegStatus != 0)
	{
		logComment("C3DMoz registerPLID: Could not create the suffixes value, "+suffix+", as expected. Errorcode="+c3dRegStatus);
		return c3dRegStatus;
	}

	/////////////////////////////////
	// Write the subkey Suffixes  //
	///////////////////////////////

	var suffixPath = c3dMozillaPluginPath+"\\Suffixes";

	// Write the suffix of the mimetype
	c3dRegStatus = winreg.createKey(suffixPath, "");
	if (c3dRegStatus != 0)
	{
		logComment("C3DMoz registerPLID: Could not create the suffix key, "+suffixPath+", as expected. ErrorCode="+c3dRegStatus);
		return c3dRegStatus;
	}

	// Write the suffix (extension), (one value-key with no value)
	c3dRegStatus = winreg.setValueString(suffixPath, suffix, "\0");
	if (c3dRegStatus != 0)
	{
		logComment("C3DMoz registerPLID: Could not create the suffix value. RegPath="+suffixPath+", value="+suffix+", as expected. Errorcode="+c3dRegStatus);
		return c3dRegStatus;
	}

	logComment("C3DMoz registerPLID: Registry keys seems to be written successfully");

	return 0;
}
